;================================
;          ZX-Assembler-2 
;      by Artic Computing LTD
;================================

ORG 5014

@LADDR=09D8
@DFILE=400C
@DFPP=400E
@LastK=4025			;Last key sysvar
@Seed=4032			;Puzzle pointer

@JumpTable
	JR Enlarge		;20500
	JR PuzzleSet		;20502
	JR Invert		;20504
	JP Release		;20506
	JP Press		;20509

@Char				;Character to print, 1 to 16
	00
@Puzzle				;Puzzle number
	00
@PAddr				;Puzzle data address
	00 00
@Pixels				;Pixel data table

        00 87 04 83 02 85 06 81
        01 86 05 82 03 84 07 80
;================================
;Find then store location of the
;puzzle address in REM line 9900
;================================
@PuzzleSet
	LD HL,+9900		;Line number 9900
	CALL LADDR		;Find it's address
        LD DE,0007        	;Skip header bytes
	ADD HL,DE
	LD E,80			;DE=128, bytes/puzzle
	LD A,(Puzzle)		;Get puzzle number
@NextPuzzle
	DEC A
	JR Z StorePuzzle
	ADD HL,DE
	JR NextPuzzle
@StorePuzzle
	LD (PAddr),HL		;Store puzzle base address
	RET
;================================
;Print enlarged character
;================================
@Enlarge
	LD A,(Char) 		;Character number to print
	AND A 			;Clear CY
	RLA			;Chr*8
	RLA
	RLA
	LD D,00
	LD E,A 			;DE=chr. code*8
	LD HL,(PAddr)		;Puzzle base address
	ADD HL,DE 		;DE=address of character pattern
	LD C,04 		;Set row counter
@Row2
	LD B,04 		;Set bit counter
	LD D,(HL) 		;Transfer two pattern bytes to DE
	INC HL 
	LD E,(HL) 
	INC HL 			;Point to next pattern
	PUSH HL 		;PUSH the pointer
@TwoBits
	XOR A 			;Clear A and CY
	RL D 			;Determine "PIXEL" value
	RLA 
	RL D 
	RLA 
	RL E 
	RLA 
	RL E 
	RLA 
	LD HL,Pixels 		;Pixels base address
	ADD A,L 		;Point to graphic symbol
	LD L,A 
	LD A,(HL) 		;Fetch the graphic symbol
	LD HL,(DFPP)		;Point to current print position
	LD (HL),A 		;Print symbol
	INC HL 			;Next print position
	LD (DFPP),HL		;SET DF-CC
	DJNZ TwoBits		;Repeat until B is zero
	PUSH DE 		;PUSH the pattern bytes
	LD DE,001D 		;Set the next print position
	ADD HL,DE 
	LD (DFPP),HL 		;SET DF-CC
	POP DE 			;POP the pattern bytes
	POP HL 			;POP the pattern pointer
	DEC C 			;Decrease the row counter
	JR NZ Row2		;If not zero then next two rows of pattern
	LD DE,FF80		;Set print position for next enlarged character 
	LD HL,(DFPP)		;GET DF-CC
	ADD HL,DE		;Set next horizontal print position 
	LD (DFPP),HL		;SET DF-CC
	RET
;================================
;Invert puzzle
;================================
@Invert
	LD B,80			;128 bytes to compliment
	LD HL,(PAddr)
@InvLoop
	LD A,(HL)
	CPL
	LD (HL),A
	INC HL
	DJNZ InvLoop
	RET
;================================
;Wait for key to be released
;================================
@Release
	LD A,(LastK)		;Check LastK sysvar
	CP FF			;Key down?
	JR NZ Release		;Yes, wait for key release
	LD BC,0000		;RET 0
	RET
;================================
;Wait for keypress
;================================
@Press
	LD A,(LastK)		;Check LastK sysvar
	CP FF			;Key down?
	JR Z Press		;No, wait for keypress
	LD BC,0000		;RET 0
	RET